###################################################################################################
# _charformatmanager.py
#
# $Id:$
# $Name:$
# $Author:$
# $Revision:$
#
# Implementation of class CharFormatManager (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from __future__ import nested_scopes
from Globals import HTMLFile
import ZPublisher.HTTPRequest
import copy
import urllib
# Product Imports.
import _blobfields
import _globals


# -------------------------------------------------------------------------------------------------
#  _charformatmanager.importXml
# -------------------------------------------------------------------------------------------------

def _importXml(self, item):
  fmts = getCharFormats(self)
  id = item['key']
  dict = item['value']
  if id in fmts:
    i = fmts.index(id)
    fmts[i+1] = dict
  else:
    fmts.append(id)
    fmts.append(dict)
  self.setConfProperty('ZMS.custom.charformats',copy.deepcopy(fmts))

def importXml(self, xml):
  value = self.parseXmlString(xml)
  if type(value) is list:
    for item in value:
      _importXml(self,item)
  else:
    _importXml(self,value)


# ------------------------------------------------------------------------------------------
#  _charformatmanager.delCharFormat:
# ------------------------------------------------------------------------------------------
def delCharFormat(self, id):
  fmts = getCharFormats(self)
  i = fmts.index(id)
  del fmts[i]
  del fmts[i]
  self.setConfProperty('ZMS.custom.charformats',copy.deepcopy(fmts))


# ------------------------------------------------------------------------------------------
#  _charformatmanager.setCharFormat:
# ------------------------------------------------------------------------------------------
def setCharFormat(self, id, newId, newDisplay, newBtn, newTag='', newAttrs=''):
  fmts = getCharFormats(self)
  if id in fmts:
    i = fmts.index(id)
  else:
    i = len(fmts)
    fmts.append(newId)
    fmts.append({'display':{}})
  dict = fmts[i+1]
  dict['display'] = newDisplay
  dict['btn'] = newBtn
  dict['tag'] = newTag
  dict['attrs'] = newAttrs
  fmts[i] = newId
  fmts[i+1] = dict.copy()
  self.setConfProperty('ZMS.custom.charformats',copy.deepcopy(fmts))


# ------------------------------------------------------------------------------------------
#  _charformatmanager.getCharFormats:
# ------------------------------------------------------------------------------------------
def getCharFormats(self):
  return self.getConfProperty('ZMS.custom.charformats',[])


###################################################################################################
###################################################################################################
###
###   class CharFormatManager:
###
###################################################################################################
###################################################################################################
class CharFormatManager: 


    # ------------------------------------------------------------------------------------------
    #  CharFormatManager.getCharFormats:
    # ------------------------------------------------------------------------------------------
    def getCharFormats(self):
      return getCharFormats(self.getDocumentElement())


    ############################################################################################
    #  CharFormatManager.manage_customizeCharFormat:
    #
    #  Change char-formats.
    ############################################################################################
    def manage_customizeCharFormat(self, lang, manage_lang, REQUEST, RESPONSE): 
      """ CharFormatManager.manage_customizeCharFormat """
      message = ''
      id = REQUEST.get('id','')
      
      # Change.
      # -------
      if REQUEST['btn'] == self.getLangStr('BTN_CHANGE',manage_lang):
        old_id = REQUEST['id']
        id = REQUEST['new_id'].strip()
        display = REQUEST['new_display'].strip()
        btn = REQUEST.get('new_btn','')
        if isinstance(btn,ZPublisher.HTTPRequest.FileUpload):
          if len(getattr(btn,'filename',''))==0:
            btn = item.get('btn',None)
          else:
            btn = _blobfields.createBlobField(self,_globals.DT_IMAGE,btn)
        tag = REQUEST['new_tag'].strip()
        attrs = REQUEST['new_attrs'].strip()
        setCharFormat(self,old_id,id,display,btn,tag,attrs)
        id = ""
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      # Delete.
      # -------
      elif REQUEST['btn'] == self.getLangStr('BTN_DELETE',manage_lang):
        id = REQUEST['id']
        delCharFormat(self,id) 
        id = ""
        message = self.getLangStr('MSG_DELETED',manage_lang)%int(1)
      
      # Insert.
      # -------
      elif REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
        id = REQUEST['_id'].strip()
        display = REQUEST['_display'].strip()
        btn = REQUEST.get('_btn','')
        if isinstance(btn,ZPublisher.HTTPRequest.FileUpload):
          if len(getattr(btn,'filename',''))==0:
            btn = item.get('btn',None)
          else:
            btn = _blobfields.createBlobField(self,_globals.DT_IMAGE,btn)
        setCharFormat(self,'',id,display,btn)
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      # Export.
      # -------
      elif REQUEST['btn'] == self.getLangStr('BTN_EXPORT',manage_lang):
        value = []
        ids = REQUEST.get('ids',[])
        fmts = getCharFormats(self)
        for i in range(len(fmts)/2):
          id = fmts[i*2]
          ob = fmts[i*2+1]
          if id in ids or len(ids) == 0:
            value.append({'key':id,'value':ob})
        if len(value)==1:
          value = value[0]
        content_type = 'text/xml; charset=utf-8'
        filename = 'export.textfmt.xml'
        export = self.getXmlHeader() + self.toXmlString(value,1)
        RESPONSE.setHeader('Content-Type',content_type)
        RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
        return export

      # Import.
      # -------
      elif REQUEST['btn'] == self.getLangStr('BTN_IMPORT',manage_lang):
        importXml(self,xml=REQUEST['file'])
        message = self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%REQUEST['file'].filename)

      # Return with message.
      message = urllib.quote(message)
      return RESPONSE.redirect('manage_customizeTextFormatForm?lang=%s&manage_lang=%s&manage_tabs_message=%s&id=%s'%(lang,manage_lang,message,id))

###################################################################################################
